#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "gauss.h"
#include "moves.h"
#include "poly.h"

typedef struct intlist { // linked list for storing codes associated with poly
  struct intlist *next;
  int n;
} intlist;

typedef struct polytree { // binary tree for storing polynomials
  struct polytree *left, *right;
  poly p;
  intlist *codes;
} polytree;

void addtotree(polytree *, poly, int);
int printtree(polytree *, code *, int);

//int nforcheck;

int main(int argc, char *argv[]) {
  if (argc < 3) {
    printf("Usage: countpolys <max crossings> <number of crossings for output> [number of strands] [0 for non-cyclic] [1 for reversible]\n");
    return 0;
  }
  int sz = atoi(argv[1]);
  int maxp = atoi(argv[2]);
  int ns = (argc >= 4 ? atoi(argv[3]) : 1);
  cyclic = (argc >= 5 ? atoi(argv[4]) : 1);
  reversible = (argc == 6 ? atoi(argv[5]) : 0);
  initcrossings((2 * ns * ns - ns) * sz);
  initpascal((2 * ns * ns - ns) * sz + 2);
  int n;
  code *codes = genallcodes(maxp, &n);
  //int *lowest = findlowest(codes, n, sz);
  //initlowest(FILENAME, 0);
  initlowest(sz);
//  nforcheck = n >> maxp;
  n <<= sz - maxp;
  srand(time(0));

  int i, m = 0;
  polytree t;
  t.p.n = -1;
  t.left = t.right = NULL;
  for (i = 0; i < n; i++)
    if (getlowest(i) == -1) {
      fprintf(stderr, "%d of %d\r", i, n);
      code c = getcode(codes, sz, i);
      if (c.n != -1) {
	if (c.n <= maxp) {
	  m++;
	  //fflush(stdout);
	  poly p = multijonespoly(c, ns);
	  if (t.p.n == -1) { // first one
	    t.p = p;
	    t.codes = calloc(1, sizeof(intlist));
	    t.codes->n = i;
	  } else
	    addtotree(&t, p, i);
	  //free(p.coeffs);
	}
	free(c.code);
      }
    }
  printf("                   \r");
  int j = printtree(&t, codes, sz);
  printf("%d unique codes, with %d different polynomials\n", m, j);
  return 0;
}

void addtotree(polytree *t, poly p, int n) {
  int cmp = comparepoly(p, t->p);
  if (cmp == 0) {
    intlist *l = t->codes;
    while (l->next != NULL)
      l = l->next;
    l->next = malloc(sizeof(intlist));
    l = l->next;
    l->next = NULL;
    l->n = n;
    free(p.coeffs); // this is only called by main(), and it's appropriate here
    return;
  } else if (cmp == -1) {
    if (t->left != NULL) {
      addtotree(t->left, p, n);
      return;
    } else {
      t->left = calloc(1, sizeof(polytree));
      t = t->left;
    }
  } else {
    if (t->right != NULL) {
      addtotree(t->right, p, n);
      return;
    } else {
      t->right = calloc(1, sizeof(polytree));
      t = t->right;
    }
  }
  t->p = p;
  t->codes = malloc(sizeof(intlist));
  t->codes->next = NULL;
  t->codes->n = n;
}

int printtree(polytree *t, code *codes, int sz) { // returns number of polys
  if (t == NULL)
    return 0;
  int n = 1;
  n += printtree(t->left, codes, sz);
  printpoly(t->p);
  printf("\n");
  intlist *l;
  /*  if (t->codes->next != NULL) {
    code c1 = getcode(codes, sz, t->codes->n);
    code c2 = getcode(codes, sz, t->codes->next->n);
    code m;
    m.n = c1.n;
    m.code = malloc(2 * m.n * sizeof(codeelem));
    int i;
    for (i = 0; i < 2 * m.n; i++)
      m.code[i] = (c1.code[i] / 6) * 6 + 3 *
	(crossings[c1.code[i]].p == O ? U : O) + c1.code[i] % 3;
    code r = first(m);
    knot rk = find(r, codes, 0, nforcheck);
    if (rk == -1) {
      printf("RK == -1 !!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
      return n;
    }
    rk <<= sz;
    rk |= getsigns(r);
    knot rkl = getlowest(rk) == -1 ? rk : getlowest(rk);
    //if (compare(r, c2) == 0)
    if (rkl == t->codes->next->n)
      printf("Codes are double mirrors of each other\n");
    else
      printf("Not double mirrors\n");
    if (r.code != m.code)
      free(m.code);
    free(r.code);
    free(c1.code);
    free(c2.code);
    }*/
  for (l = t->codes; l != NULL; l = l->next) {
    code c = getcode(codes, sz, l->n);
    //printf("%d \t", Jinvar(c));
    printf("\t");
    printcode(c);
    free(c.code);
  }
  n += printtree(t->right, codes, sz);
  return n;
}
