#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include "gauss.h"
#include "moves.h"
#include "pd.h"
#include "poly.h"
//#include "draw.h"
#include "polytrees.h"
#include "writehtml.h"

int main(int argc, char *argv[]) {
  if (argc < 3) {
    printf("Usage: knotinfo <max crossings> <number of crossings in knot> [0 for non-cyclic] [1 for reversible]\n");
    return 0;
  }
  int sz = atoi(argv[1]);
  int maxp = atoi(argv[2]);
  cyclic = (argc >= 4 ? atoi(argv[3]) : 1);
  reversible = (argc == 5 ? atoi(argv[4]) : 0);
  initcrossings(50 * maxp);
  initpascal(52 * maxp);
  knot n;
  code *codes = genallcodes(maxp, &n);
  //int *lowest = findlowest(codes, n, sz);
  //findlowest(codes, n, sz);
  //initlowest(FILENAME, 0);
  initlowest(sz);
  n <<= sz - maxp;

  knot i, m = 0, j = 0;
  int nc = -1, nk = 1;
  kname names[n];
  for (i = 0; i < n; i++) {
    names[i].nc = -1;
    names[i].nk = -1;
    code c = getcode(codes, sz, i);
    if (c.n != -1) {
      if (c.n <= maxp) {
	knot l = getlowest(i);
	if (l == -1) {
	  //fprintf(stderr, "%d of %d\r", i, n);

	  knot f = getfirst(i, codes, n, sz);
	  if (f < i) {
	    names[i] = names[f];
	    free(c.code);
	    continue;
	  }

	  if (c.n > nc) {
	    nc = c.n;
	    nk = 1;
	  }

	  names[i].nc = nc;
	  names[i].nk = nk;
	  nk++;

	  j++;
	}
      }
      free(c.code);
    }
  }

  char line[1024];
  printf("Enter Gauss code: ");
  if (gets(line) == NULL)
    return 1;
  code c = parsecode(line);
  code f = simplify(c);
  printf("Gauss code used in enumeration: ");
  printcode(f);
  knot l = getknot(f, codes, n, sz);
  code cl = getcode(codes, sz, l);
  printf("Base Gauss code: ");
  printcode(cl);
  knot kf = getfirst(l, codes, n, sz);
  code cf = getcode(codes, sz, kf);
  printf("Base mirror image/inverse: ");
  printcode(cf);
  if (compare(cl, cf) == 0)
    printf("(No operation needed to get the base)\n");
  else if (compare(hmirror(cl), cf) == 0)
    printf("(Horizontal mirror image)\n");
  else if (compare(vmirror(cl), cf) == 0)
    printf("(Vertical mirror image)\n");
  else if (compare(doublemirror(cl), cf) == 0)
    printf("(Double mirror)\n");
  else if (compare(inverse(cl), cf) == 0)
    printf("(Inverse)\n");
  else if (compare(inverse(hmirror(cl)), cf) == 0)
    printf("(Inverse of horizontal mirror image)\n");
  else if (compare(inverse(vmirror(cl)), cf) == 0)
    printf("(Inverse of vertical mirror image)\n");
  else if (compare(inverse(doublemirror(cl)), cf) == 0)
    printf("(Inverse of double mirror)\n");
  else
    printf("An error has occurred: no relationship found!\n");
  printf("Virtual knot number: %d.%d\n", names[kf].nc, names[kf].nk);
  PD pd = code2PD(c);
  printf("PD: "); printPD(pd);
  static int maxjones[] = {5, 5, 5, 4, 3, 3, 2, 2, 2};
  poly p = jonespoly2(cl);
  printf("Jones polynomial: "); printpoly(p); printf("\n");
  for (i = 2; i <= maxjones[cl.n]; i++) {
    printf("%d-cabled Jones polynomial: ", i);
    p = multijonespoly(cl, i);
    printpoly(p);
    printf("\n");
  }
  twopoly tp = genalexander(cl);
  printf("Generalized Alexander polynomial: "); printtwopoly(tp); printf("\n");
  printf("Self-linking number: %d\n", Jinvar(c));

  return 0;
}
