#include <stdio.h>
#include <stdlib.h>
#include "gauss.h"
#include "moves.h"
#include "poly.h"
#include "pd.h"
#include "polytrees.h"
#include "writehtml.h"

int cmpkname(kname ka, kname kb) {
  if (ka.nc < kb.nc)
    return -1;
  if (ka.nc > kb.nc)
    return 1;
  if (ka.nk < kb.nk)
    return -1;
  if (ka.nk > kb.nk)
    return 1;
  return 0;
}

int vcmpkname(const void *a, const void *b) {
  return cmpkname(*((kname *)a), *((kname *)b));
}

void writeHTML(code c, kname name, kname prev, kname next, int isfirst,
	       kname *names, polytree *tjones,
	       polytree *tjones3, twopolytree *tgenalex) {
  static int maxcables[] = {5, 5, 5, 4, 3, 2, 2, 2};
  char filename[1024];
  snprintf(filename, 1023, "html/%d.%d.html", name.nc, name.nk);
  FILE *f = fopen(filename, "w");

  fprintf(f, "<!DOCTYPE html PUBLIC \"-//W3C/DTD XHTML 1.0 Strict //EN\""
	  " \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
	  "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\""
	  " lang=\"en\">\n"
	  "<head>\n"
	  "<title>%d.%d</title>\n"
	  "<link href=\"content.css\" rel=\"stylesheet\" type=\"text/css\" />"
	  "</head>\n"
	  "<body>\n"
	  "<table class=\"navbar\"><tr>\n", name.nc, name.nk);
  if (prev.nc != -1)
    fprintf(f, "<td><a href=\"%d.%d.html\"><div><a href=\"%d.%d.html\">"
	    "<img src=\"images/%d.%d-s.png\" /></a><br />"
	    "<a href=\"%d.%d.html\">%d.%d</a></div>"
	    "</a></td>\n", prev.nc, prev.nk, prev.nc, prev.nk,
	    prev.nc, prev.nk, prev.nc, prev.nk, prev.nc, prev.nk);
  fprintf(f, "<td><a href=\"index.html\"><div>Index</div></a></td>\n");
  if (next.nc != -1)
    fprintf(f, "<td><a href=\"%d.%d.html\"><div><a href=\"%d.%d.html\">"
	    "<img src=\"images/%d.%d-s.png\" /></a><br />"
	    "<a href=\"%d.%d.html\">%d.%d</a></div>"
	    "</a></td>\n", next.nc, next.nk, next.nc, next.nk, 
	    next.nc, next.nk, next.nc, next.nk, next.nc, next.nk);
  fprintf(f, "</tr></table>\n"
	  "<h1>Virtual Knot %d.%d</h1>\n"
	  "<img src=\"images/%d.%d.png\" />\n"
	  "<dl>\n", name.nc, name.nk, name.nc, name.nk);

  fprintf(f,"<dt>This virtual knot is %sequivalent to its vertical <a "
	  "href=\"mirrorimages.html\">mirror image</a>.</dt>\n",
	  isfirst & 1 ? "not " : "");
  fprintf(f, "<dt>This virtual knot is %sequivalent to its horizontal <a "
	  "href=\"mirrorimages.html\">mirror image</a>.</dt>\n",
	  isfirst & 2 ? "not " : "");
  if ((isfirst & 3) == 3)
    fprintf(f,"<dt>The two mirror images are %sequivalent to each other."
	    "</dt>\n", isfirst & 4 ? "not " : "");
  fprintf(f, "<dt>This virtual knot is %sequivalent to its "
	  "<a href=\"inverse.html\">inverse</a>.</dt>\n",
	  isfirst & 8 ? "not " : "");

  char *s = sprintcode(c);
  fprintf(f, "<dt><a href=\"gausscode.html\">Gauss code</a></dt><dd>%s</dd>\n",
	  s);
  free(s);
  PD pd = code2PD(c);
  s = sprintPD(pd);
  fprintf(f, "<dt><a href=\"pd.html\">PD presentation</a></dt>"
	  "<dd>PD[%s]</dd>\n", s);
  freePD(pd);
  free(s);

  poly jones = jonespoly2(c);
  s = sprintjones(jones);
  fprintf(f, "<dt><a href=\"jones.html\">Jones polynomial</a></dt>"
	  "<dd>%s</dd>\n", s);
  free(s);

  fprintf(f, "<dt>Other virtual knots (up to mirrors) with the same Jones "
	  "polynomial</dt>\n<dd>");
  knotlist *kl1 = pgetknots(tjones, jones);
  poly jonesm = jonesmirror(jones);
  knotlist *kl2 = pgetknots(tjones, jonesm);
  free(jones.coeffs);
  free(jonesm.coeffs);
  knotlist *kl;
  int i;
  for (i = 0, kl = kl1; kl != NULL; i++, kl = kl->next);
  for (kl = kl2; kl != NULL; i++, kl = kl->next);
  kname *others = malloc(i * sizeof(kname));
  for (i = 0, kl = kl1; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  for (kl = kl2; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  qsort(others, i, sizeof(kname), vcmpkname);
  kname last = {0, 0};
  int j;
  int output = 0;
  for (j = 0; j < i; j++)
    if ((others[j].nc != last.nc || others[j].nk != last.nk) &&
	(others[j].nc != name.nc || others[j].nk != name.nk)) {
      if (last.nk != 0)
	fprintf(f, ", ");
      last = others[j];
      fprintf(f, "<a href=\"%d.%d.html\">%d.%d</a>", others[j].nc,
	      others[j].nk, others[j].nc, others[j].nk);
      output = 1;
    }
  if (!output)
    fprintf(f, "none");
  fprintf(f, "</dd>\n");
  free(others);

  fprintf(f, "<dt><a href=\"jones.html\">Cabled Jones polynomials</a></dt>"
	  "<dd><ol start=\"2\">\n");
  poly jones3, p;
  for (i = 2; i <= maxcables[c.n]; i++) {
    p = multijonespoly(c, i);
    s = sprintjones(p);
    fprintf(f, "<li>%s</li>\n", s);
    if (i == 3)
      jones3 = p;
    else
      free(p.coeffs);
    free(s);
  }
  fprintf(f, "</ol></dd>\n");

  fprintf(f, "<dt>Other virtual knots (up to mirrors) with the same cabled "
	  "Jones polynomials</dt>\n<dd>");
  kl1 = pgetknots(tjones3, jones3);
  poly jones3m = jonesmirror(jones3);
  kl2 = pgetknots(tjones3, jones3m);
  free(jones3.coeffs);
  free(jones3m.coeffs);
  //knotlist *kl;
  for (i = 0, kl = kl1; kl != NULL; i++, kl = kl->next);
  for (kl = kl2; kl != NULL; i++, kl = kl->next);
  others = malloc(i * sizeof(kname));
  for (i = 0, kl = kl1; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  for (kl = kl2; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  qsort(others, i, sizeof(kname), vcmpkname);
  last.nc = 0;
  last.nk = 0;
  //int j;
  output = 0;
  for (j = 0; j < i; j++)
    if ((others[j].nc != last.nc || others[j].nk != last.nk) &&
	(others[j].nc != name.nc || others[j].nk != name.nk)) {
      if (last.nk != 0)
	fprintf(f, ", ");
      last = others[j];
      fprintf(f, "<a href=\"%d.%d.html\">%d.%d</a>", others[j].nc,
	      others[j].nk, others[j].nc, others[j].nk);
      output = 1;
    }
  if (!output)
    fprintf(f, "none");
  fprintf(f, "</dd>\n");
  free(others);

  twopoly tp = genalexander(c);
  s = sprinttwopoly(tp);
  fprintf(f, "<dt><a href=\"genalexander.html\">Generalized Alexander "
	  "polynomial</a></dt><dd>%s</dd>\n", s);
  //freetwopoly(tp);
  free(s);

  fprintf(f, "<dt>Other virtual knots (up to mirrors/inversion) with the same "
	  "Generalized Alexander polynomial</dt>\n<dd>");
  kl1 = tpgetknots(tgenalex, tp);
  twopoly tp2 = genalexhmirror(tp);
  kl2 = tpgetknots(tgenalex, tp2);
  twopoly tp3 = genalexvmirror(tp);
  knotlist *kl3 = tpgetknots(tgenalex, tp3);
  twopoly tp4 = genalexvmirror(tp2);
  knotlist *kl4 = tpgetknots(tgenalex, tp4);
  freetwopoly(tp); freetwopoly(tp2); freetwopoly(tp3); freetwopoly(tp4);
  for (i = 0, kl = kl1; kl != NULL; i++, kl = kl->next);
  for (kl = kl2; kl != NULL; i++, kl = kl->next);
  for (kl = kl3; kl != NULL; i++, kl = kl->next);
  for (kl = kl4; kl != NULL; i++, kl = kl->next);
  others = malloc(i * sizeof(kname));
  for (i = 0, kl = kl1; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  for (kl = kl2; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  for (kl = kl3; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  for (kl = kl4; kl != NULL; i++, kl = kl->next)
    others[i] = names[kl->n];
  qsort(others, i, sizeof(kname), vcmpkname);
  last.nc = 0; last.nk = 0;
  output = 0;
  for (j = 0; j < i; j++)
    if ((others[j].nc != last.nc || others[j].nk != last.nk) &&
	(others[j].nc != name.nc || others[j].nk != name.nk)) {
      if (last.nk != 0)
	fprintf(f, ", ");
      last = others[j];
      fprintf(f, "<a href=\"%d.%d.html\">%d.%d</a>", others[j].nc,
	      others[j].nk, others[j].nc, others[j].nk);
      output = 1;
    }
  if (!output)
    fprintf(f, "none");
  fprintf(f, "</dd>\n");
  free(others);

  fprintf(f, "<dt><a href=\"selflinking.html\">Self linking number</a></dt>"
	  "<dd>%d</dd>", Jinvar(c));

  fprintf(f, "</dl>\n</body>\n</html>\n");
  fclose(f);
}
